# requirement
# original link
# https://github.com/Isabek/XmlToTxt

"""
declxml==0.9.1
"""

"""
xml and txt are two formats of object detection labels, they are same labels just in different format

different models take different format as input

"""

import sys
import logging
import os
import declxml as xml


class ObjectMapper(object):
    def __init__(self):
        self.processor = xml.user_object("annotation", Annotation, [
            xml.user_object("size", Size, [
                xml.integer("width"),
                xml.integer("height"),
            ]),
            xml.array(
                xml.user_object("object", Object, [
                    xml.string("name"),
                    xml.user_object("bndbox", Box, [
                        xml.floating_point("xmin"),
                        xml.floating_point("ymin"),
                        xml.floating_point("xmax"),
                        xml.floating_point("ymax"),
                    ], alias="box")
                ]),
                alias="objects"
            ),
            xml.string("filename")
        ])

    def bind(self, xml_file_path, xml_dir):
        ann = xml.parse_from_file(self.processor, xml_file_path=os.path.join(xml_dir, xml_file_path))
        ann.filename = xml_file_path
        return ann

    def bind_files(self, xml_file_paths, xml_dir):
        result = []
        for xml_file_path in xml_file_paths:
            try:
                result.append(self.bind(xml_file_path=xml_file_path, xml_dir=xml_dir))
            except Exception as e:
                logging.error("%s", e.args)
        return result


class Annotation(object):
    def __init__(self):
        self.size = None
        self.objects = None
        self.filename = None

    def __repr__(self):
        return "Annotation(size={}, object={}, filename={})".format(self.size, self.objects, self.filename)


class Size(object):
    def __init__(self):
        self.width = None
        self.height = None

    def __repr__(self):
        return "Size(width={}, height={})".format(self.width, self.height)


class Object(object):
    def __init__(self):
        self.name = None
        self.box = None

    def __repr__(self):
        return "Object(name={}, box={})".format(self.name, self.box)


class Box(object):
    def __init__(self):
        self.xmin = None
        self.ymin = None
        self.xmax = None
        self.ymax = None

    def __repr__(self):
        return "Box(xmin={}, ymin={}, xmax={}, ymax={})".format(self.xmin, self.ymin, self.xmax, self.ymax)


class Reader(object):
    def __init__(self, xml_dir):
        self.xml_dir = xml_dir

    def get_xml_files(self):
        xml_filenames = []
        for root, subdirectories, files in os.walk(self.xml_dir):
            for filename in files:
                if filename.endswith(".xml"):
                    file_path = os.path.join(root, filename)
                    file_path = os.path.relpath(file_path, start=self.xml_dir)
                    xml_filenames.append(file_path)
        return xml_filenames

    @staticmethod
    def get_classes(filename):
        with open(os.path.join(os.path.dirname(os.path.realpath('__file__')), filename), "r", encoding="utf8") as f:
            lines = f.readlines()
            return {value: key for (key, value) in enumerate(list(map(lambda x: x.strip(), lines)))}


class Transformer(object):
    def __init__(self, xml_dir, out_dir):
        self.xml_dir = xml_dir
        self.out_dir = out_dir
        self.all_classes = list()
        self.label_map = dict()

    def transform(self):
        reader = Reader(xml_dir=self.xml_dir)
        xml_files = reader.get_xml_files()
        object_mapper = ObjectMapper()
        annotations = object_mapper.bind_files(xml_files, xml_dir=self.xml_dir)
        self.write_to_txt(annotations)

    def write_to_txt(self, annotations):
        for annotation in annotations:
            output_path = os.path.join(self.out_dir, self.darknet_filename_format(annotation.filename))
            if not os.path.exists(os.path.dirname(output_path)):
                os.makedirs(os.path.dirname(output_path))
            with open(output_path, "w+") as f:
                f.write(self.to_darknet_format(annotation))

    def to_darknet_format(self, annotation):
        result = []
        for obj in annotation.objects:
            isin =  obj.name in self.all_classes
            if not isin:
                self.all_classes.append(obj.name)
            int_label = self.all_classes.index(obj.name)
            self.label_map[int_label] = obj.name

            x, y, width, height = self.get_object_params(obj, annotation.size)
            result.append("%d %.6f %.6f %.6f %.6f" % (int_label, x, y, width, height))
        return "\n".join(result)

    @staticmethod
    def get_object_params(obj, size):
        image_width = 1.0 * size.width
        image_height = 1.0 * size.height

        box = obj.box
        absolute_x = box.xmin + 0.5 * (box.xmax - box.xmin)
        absolute_y = box.ymin + 0.5 * (box.ymax - box.ymin)

        absolute_width = box.xmax - box.xmin
        absolute_height = box.ymax - box.ymin

        x = absolute_x / image_width
        y = absolute_y / image_height
        width = absolute_width / image_width
        height = absolute_height / image_height

        return x, y, width, height

    @staticmethod
    def darknet_filename_format(filename):
        pre, ext = os.path.splitext(filename)
        return "%s.txt" % pre


def convert(xml_dir, out_dir):
    """

    @param xml_dir:
    @param out_dir:
    @return:
    """
    if not os.path.exists(xml_dir):
        print("Provide the correct folder for xml files.")
        sys.exit()

    if not os.path.exists(out_dir):
        os.makedirs(out_dir)

    if not os.access(out_dir, os.W_OK):
        print("%s folder is not writeable." % out_dir)
        sys.exit()

    transformer = Transformer(xml_dir=xml_dir, out_dir=out_dir)
    transformer.transform()

    return transformer.label_map


def demo():
    xml_data_dir = 'linmao_camera_data'
    output_txt_data_dir = 'linmao_camera_data_txt'
    label_map = convert(xml_data_dir, output_txt_data_dir)
    print(label_map)



import os
import json
import numpy as np
from scipy.interpolate import interp1d
from scipy.signal import savgol_filter

abs_path = os.path.dirname(os.path.abspath(__file__))


class training_pred_class:
    def __init__(self):
        self.data_points = 200
        self.smooth_window_size = 21
        self.smooth_polynomial_order = 3
        with open(os.path.join(abs_path, 'data', 'trainpredict.json')) as f:
            self.data = json.load(f)
        self.look_up_table = dict((int(k), v) for k, v in self.data['look_up_table'].items())

    @staticmethod
    def get_features_from_keras_model(keras_model_obj):
        layers = [i for i in keras_model_obj.get_config()['layers'] if i['class_name'] == 'Dense']
        return layers

    def predict(self, model, batch_size, optimizer, GPU_TYPE='P100'):
        data_points = self.data_points
        smooth_window_size = self.smooth_window_size
        smooth_polynomial_order = self.smooth_polynomial_order
        look_up_table = self.look_up_table

        filename = f'new_{optimizer}_batch_{batch_size}'

        data_dict = self.data[f'AggregateExp_NewModel_{GPU_TYPE}_{filename}']

        all_layer_batch = data_dict['all_layer_batch'][:data_points]
        layer_numbers = data_dict['layer_numbers'][:data_points]

        xdata = np.array(layer_numbers).astype(np.float)
        ydata = np.array([np.median(i) for i in all_layer_batch])

        y_smooth = savgol_filter(
            ydata, smooth_window_size, smooth_polynomial_order
        )  # window size 51, polynomial order 3
        f_interp1d = interp1d(xdata, y_smooth, kind='cubic')

        features = training_pred_class.get_features_from_keras_model(model)

        layer_batch_times = []
        for feature in features:
            Lbatch = look_up_table[batch_size][optimizer]
            layer_batch_times.append(Lbatch)

        sum_layers_batch = sum(layer_batch_times)
        multiplier = float(f_interp1d(len(features)))

        pred = sum_layers_batch / multiplier
        return pred


prediction_model = training_pred_class()
